searchData={"items":[{"type":"module","title":"EctoNestedChangeset","doc":"This module defines function for manipulating nested changesets.\n\nAll functions take a path as the second argument. The path is a list of atoms\n(for field names) and integers (for indexes in lists).","ref":"EctoNestedChangeset.html"},{"type":"function","title":"EctoNestedChangeset.append_at/3","doc":"Appends a value to the field referenced by the path.\n\nThe last path segment must be an atom referencing either a to-many relation\nfield or an array field.","ref":"EctoNestedChangeset.html#append_at/3"},{"type":"function","title":"Example - EctoNestedChangeset.append_at/3","doc":"iex> %Owner{pets: [%Pet{}, %Pet{toys: [%Toy{name: \"stick\"}]}]}\n    ...> |> Ecto.Changeset.change()\n    ...> |> append_at(changeset, [:pets, 1, :toys], %Toy{name: \"ball\"})\n    ...> |> Ecto.Changeset.apply_changes()\n    %Owner{\n      pets: [\n        %Pet{},\n        %Pet{toys: [%Toy{name: \"stick\"}, %Toy{name: \"ball\"}]}\n      ]\n    }","ref":"EctoNestedChangeset.html#append_at/3-example"},{"type":"function","title":"EctoNestedChangeset.delete_at/3","doc":"Deletes the item at the given path.\n\nThe last path segment is expected to be an integer index.\n\nItems that are not persisted in the database yet will always be removed from\nthe list. For structs that are already persisted in the database, there are\nthree different modes.\n\n- `[mode: {:action, :replace}]` (default) - The item will be wrapped in a\n  changeset with the `:replace` action. This only works if an appropriate\n  `:on_replace` option is set for the relation in the schema.\n- `[mode: {:action, :delete}]` - The item will be wrapped in a changeset with\n  the action set to `:delete`.\n- `[mode: {:flag, field}]` - Puts `true` as a change for the given field.\n\nThe flag option useful for explicitly marking items for deletion in form\nparameters. In this case, you would configure a virtual field on the schema\nand set the changeset action to `:delete` in the changeset function in case\nthe value is set to `true`.\n\n    schema \"pets\" do\n      field :name, :string\n      field :delete, :boolean, virtual: true, default: false\n    end\n\n    def changeset(pet, attrs) do\n      pet\n      |> cast(attrs, [:name, :delete])\n      |> validate_required([:name])\n      |> maybe_mark_for_deletion()\n    end\n\n    def maybe_mark_for_deletion(%Ecto.Changeset{} = changeset) do\n      if Ecto.Changeset.get_change(changeset, :delete),\n        do: Map.put(changeset, :action, :delete),\n        else: changeset\n    end","ref":"EctoNestedChangeset.html#delete_at/3"},{"type":"function","title":"Examples - EctoNestedChangeset.delete_at/3","doc":"iex> changeset = Ecto.Changeset.change(\n           %Owner{pets: [%Pet{name: \"George\"}, %Pet{name: \"Patty\"}]}\n    ...> )\n    iex> delete_at(changeset, [:pets, 1])\n    %Ecto.Changeset{\n      changes: [\n        %Changeset{action: :replace, data: %Post{name: \"Patty\"}},\n        %Changeset{action: :update, data: %Post{name: \"George\"}},\n      ]\n    }\n    iex> delete_at(changeset, [:pets, 1], mode: {:action, :delete})\n    %Ecto.Changeset{\n      changes: [\n        %Changeset{action: :update, data: %Post{name: \"George\"}},\n        %Changeset{action: :delete, data: %Post{name: \"Patty\"}},\n      ]\n    }\n    iex> delete_at(changeset, [:pets, 1], mode: {:field, :delete})\n    %Ecto.Changeset{\n      changes: [\n        %Changeset{action: :update, data: %Post{name: \"George\"}},\n        %Changeset{\n          action: :update,\n          changes: %{delete: true},\n          data: %Post{name: \"Patty\"}\n        },\n      ]\n    }","ref":"EctoNestedChangeset.html#delete_at/3-examples"},{"type":"function","title":"EctoNestedChangeset.get_at/2","doc":"Returns a value from a changeset referenced by the path.","ref":"EctoNestedChangeset.html#get_at/2"},{"type":"function","title":"Example - EctoNestedChangeset.get_at/2","doc":"iex> %Owner{pets: [%Pet{}, %Pet{toys: [%Toy{name: \"stick\"}]}]}\n    ...> |> Ecto.Changeset.change()\n    ...> |> get_at(changeset, [:pets, 1, :toys])\n    [%Toy{name: \"stick\"}, %Toy{name: \"ball\"}]","ref":"EctoNestedChangeset.html#get_at/2-example"},{"type":"function","title":"EctoNestedChangeset.insert_at/3","doc":"Inserts a value into a field at the given position.\n\nThe last path segment must be an integer for the position.","ref":"EctoNestedChangeset.html#insert_at/3"},{"type":"function","title":"Example - EctoNestedChangeset.insert_at/3","doc":"iex> %Owner{\n    ...>   pets: [\n    ...>     %Pet{},\n    ...>     %Pet{toys: [%Toy{name: \"stick\"}, %Toy{name: \"ball\"}]}\n    ...>   ]\n    ...> }\n    ...> |> Ecto.Changeset.change()\n    ...> |> insert_at(changeset, [:pets, 1, :toys, 1], %Toy{name: \"rope\"})\n    ...> |> Ecto.Changeset.apply_changes()\n    %Owner{\n      pets: [\n        %Pet{},\n        %Pet{\n          toys: [\n            %Toy{name: \"ball\"},\n            %Toy{name: \"rope\"},\n            %Toy{name: \"stick\"}\n          ]\n        }\n      ]\n    }","ref":"EctoNestedChangeset.html#insert_at/3-example"},{"type":"function","title":"EctoNestedChangeset.prepend_at/3","doc":"Prepends a value to the field referenced by the path.\n\nThe last path segment must be an atom referencing either a to-many relation\nfield or an array field.","ref":"EctoNestedChangeset.html#prepend_at/3"},{"type":"function","title":"Example - EctoNestedChangeset.prepend_at/3","doc":"iex> %Owner{pets: [%Pet{}, %Pet{toys: [%Toy{name: \"stick\"}]}]}\n    ...> |> Ecto.Changeset.change()\n    ...> |> prepend_at(changeset, [:pets, 1, :toys], %Toy{name: \"ball\"})\n    ...> |> Ecto.Changeset.apply_changes()\n    %Owner{\n      pets: [\n        %Pet{},\n        %Pet{toys: [%Toy{name: \"ball\"}, %Toy{name: \"stick\"}]}\n      ]\n    }","ref":"EctoNestedChangeset.html#prepend_at/3-example"},{"type":"function","title":"EctoNestedChangeset.update_at/3","doc":"Updates the value in the changeset at the given position with the given update\nfunction.\n\nThe path may lead to any field, including arrays and relation fields. Unlike\n`Ecto.Changeset.update_change/3`, the update function is always applied,\neither to the change or to existing value.\n\nIf the path points to a field with a simple type, the update function will\nreceive the raw value of the field. However, if the path points to the field\nof a *-to-many relation, the list values will not be unwrapped, which means\nthat the update function has to handle a list of changesets.","ref":"EctoNestedChangeset.html#update_at/3"},{"type":"function","title":"Examples - EctoNestedChangeset.update_at/3","doc":"iex> %Owner{pets: [%Pet{toys: [%Toy{name: \"stick\"}, %Toy{name: \"ball\"}]}]}\n    ...> |> Ecto.Changeset.change()\n    ...> |> update_at(\n    ...>      changeset,\n    ...>      [:pets, 1, :toys, 1, :name],\n    ...>      &String.upcase/1\n    ...>    )\n    ...> |> Ecto.Changeset.apply_changes()\n    %Owner{\n      pets: [\n        %Pet{},\n        %Pet{\n          toys: [\n            %Toy{name: \"stick\"},\n            %Toy{name: \"BALL\"}\n          ]\n        }\n      ]\n    }","ref":"EctoNestedChangeset.html#update_at/3-examples"},{"type":"exception","title":"EctoNestedChangeset.NotLoadedError","doc":"Raised when a relation field that is updated is not preloaded.","ref":"EctoNestedChangeset.NotLoadedError.html"},{"type":"extras","title":"README","doc":"[![CI](https://github.com/woylie/ecto_nested_changeset/actions/workflows/ci.yml/badge.svg)](https://github.com/woylie/ecto_nested_changeset/actions/workflows/ci.yml) [![Hex](https://img.shields.io/hexpm/v/ecto_nested_changeset)](https://hex.pm/packages/ecto_nested_changeset) [![Coverage Status](https://coveralls.io/repos/github/woylie/ecto_nested_changeset/badge.svg?branch=main)](https://coveralls.io/github/woylie/ecto_nested_changeset?branch=main)\n\n# EctoNestedChangeset\n\nThis is a package for manipulating nested\n[Ecto](https://github.com/elixir-ecto/ecto) changesets.","ref":"readme.html"},{"type":"extras","title":"Installation - README","doc":"Add `ecto_nested_changeset` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ecto_nested_changeset, \"~> 1.0.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - README","doc":"The primary use case of this library is the manipulation of\n[Ecto](https://github.com/elixir-ecto/ecto) changesets\nused as a source for dynamic, nested forms in\n[Phoenix LiveView](https://github.com/phoenixframework/phoenix_live_view).\n\n```elixir\ncategory = %Category{\n  posts: [\n    %Post{\n      id: 1,\n      comments: [\n        %Comment{body: \"potato\", id: 1},\n        %Comment{body: \"you\", id: 2}\n      ],\n      title: \"must\"\n    },\n    %Post{comments: [], id: 2, title: \"young\"}\n  ]\n}\n\ncategory\n|> Ecto.Changeset.change()\n|> append_at(:posts, %Post{title: \"Padawan\", comments: []})\n|> prepend_at([:posts, 0, :comments], %Comment{body: \"ecneitaP\"})\n|> delete_at([:posts, 0, :comments, 1], mode: {:action, :delete})\n|> insert_at([:posts, 1], %Post{title: \"have\"})\n|> append_at([:posts, 2, :comments], %Comment{body: \"my\"})\n|> update_at([:posts, 0, :comments, 0, :body], &String.reverse/1)\n|> Ecto.Changeset.apply_changes()\n\n%Category{\n  posts: [\n    %Post{\n      comments: [\n        %Comment{body: \"Patience\"},\n        %Comment{body: \"you\", id: 2}\n      ],\n      id: 1,\n      title: \"must\"\n    },\n    %Post{title: \"have\"},\n    %Post{\n      comments: [%Comment{body: \"my\"}],\n      id: 2,\n      title: \"young\"\n    },\n    %Post{title: \"Padawan\"}\n  ]\n}\n```","ref":"readme.html#usage"},{"type":"extras","title":"Example application - README","doc":"There is an example Phoenix application with a dynamic nested LiveView form in\nthe `/example` folder of the repository.\n\n```bash\ngit clone https://github.com/woylie/ecto_nested_changeset.git\ncd ecto_nested_changeset/example\nmix setup\nmix phx.server\n```\n\nNote that Postgres needs to be running to use the application.\n\nYou can access the application at http://localhost:4000.","ref":"readme.html#example-application"},{"type":"extras","title":"Status - README","doc":"This library is actively maintained, but given its narrow purpose, it will not\nsee frequent updates.","ref":"readme.html#status"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.2"}}